Package: qvirus
Title: Quantum Computing for Analyzing CD4 Lymphocytes and
        Antiretroviral Therapy
Version: 0.0.6
Authors@R: 
    person("Juan Pablo", "Acuña González", , "22253567@uagro.mx", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0009-0003-6029-6560"))
Description: Resources, tutorials, and code snippets dedicated to exploring the intersection of quantum computing and artificial intelligence (AI) in the context of analyzing Cluster of Differentiation 4 (CD4) lymphocytes and optimizing antiretroviral therapy (ART) for human immunodeficiency virus (HIV). With the emergence of quantum artificial intelligence and the development of small-scale quantum computers, there's an unprecedented opportunity to revolutionize the understanding of HIV dynamics and treatment strategies. This project leverages a quantum computer simulator, to explore these applications in quantum computing techniques, addressing the challenges in studying CD4 lymphocytes and enhancing ART efficacy.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
LazyData: true
Suggests: rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Imports: dplyr, magrittr, stats
NeedsCompilation: no
Packaged: 2026-04-15 18:44:39 UTC; jp
Author: Juan Pablo Acuña González [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0003-6029-6560>)
Maintainer: Juan Pablo Acuña González <22253567@uagro.mx>
Repository: CRAN
Date/Publication: 2026-04-15 19:10:02 UTC
Built: R 4.5.2; ; 2026-04-17 09:25:54 UTC; unix

Package: qvirus
Title: Quantum Computing for Analyzing CD4 Lymphocytes and
        Antiretroviral Therapy
Version: 0.0.6
Authors@R: 
    person("Juan Pablo", "Acuña González", , "22253567@uagro.mx", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0009-0003-6029-6560"))
Description: Resources, tutorials, and code snippets dedicated to exploring the intersection of quantum computing and artificial intelligence (AI) in the context of analyzing Cluster of Differentiation 4 (CD4) lymphocytes and optimizing antiretroviral therapy (ART) for human immunodeficiency virus (HIV). With the emergence of quantum artificial intelligence and the development of small-scale quantum computers, there's an unprecedented opportunity to revolutionize the understanding of HIV dynamics and treatment strategies. This project leverages a quantum computer simulator, to explore these applications in quantum computing techniques, addressing the challenges in studying CD4 lymphocytes and enhancing ART efficacy.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
LazyData: true
Suggests: rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Imports: dplyr, magrittr, stats
NeedsCompilation: no
Packaged: 2026-04-15 18:44:39 UTC; jp
Author: Juan Pablo Acuña González [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0003-6029-6560>)
Maintainer: Juan Pablo Acuña González <22253567@uagro.mx>
Repository: CRAN
Date/Publication: 2026-04-15 19:10:02 UTC
Built: R 4.5.2; ; 2026-04-20 23:33:16 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(mse,InteractionClassification)
S3method(mse,payoffs)
S3method(summary,InteractionClassification)
S3method(summary,payoffs)
export(InteractionClassification)
export(cd_diff)
export(cds_diff)
export(estimate_payoffs)
export(mse)
export(nearest_payoff)
export(payoffs_list)
export(phen_hiv)
export(run_bb84_simulation)
export(run_e91_simulation)
export(summary)
export(vl_diff)
export(vlog_diff)
export(vlogs_diff)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# qvirus

<!-- badges: start -->

<!-- badges: end -->

The goal of qvirus is to provide resources, tutorials, and code snippets
dedicated to exploring the intersection of quantum computing and
artificial intelligence (AI) in the context of HIV drug design.

## Installation

You can install the development version of qvirus from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("juan-acu/qvirus")
```

#' Classify HIV phenotype interactions using k-means clustering
#'
#' This function performs k-means clustering on the differences in viral load and CD4 counts
#' to classify interaction types between HIV phenotypes. It returns an object of class
#' `InteractionClassification`, a `data.frame` with classification labels.
#'
#' @param cd_result A numeric vector of differences in CD4 T-cell counts.
#' @param vl_result A numeric vector of differences in log viral load.
#' @param k Integer. The number of clusters to use in k-means. Default is 4.
#' @param ns Integer. Number of random initializations for the k-means algorithm. Default is 100.
#' @param seed Integer. Seed for random number generation to ensure reproducibility. Default is 123.
#'
#' @returns A `data.frame` of class `InteractionClassification` with three columns:
#' \describe{
#'   \item{cds3_result}{The CD4 count difference for each interaction.}
#'   \item{vlogs3_result}{The viral load difference (log scale) for each interaction.}
#'   \item{classification}{An integer label (1 to `k`) indicating the interaction cluster.}
#' }
#' 
#' @export
#'
#' @examples
#' set.seed(42)
#' data(cd_3)
#' cd_data <- cd_3[,-1]
#' cd_result <- cds_diff(cd_data)
#' data(vl_3)
#' vl_data <- vl_3[,-1]
#' vl_result <- vlogs_diff(vl_data)
#' result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
InteractionClassification <- function(cd_result, vl_result, k = 4, ns = 100, seed = 123) {
  set.seed(seed)
  
  # Crear data frame con diferencias
  diffs <- data.frame(cd_result = cd_result, vl_result = vl_result)
  
  # Ejecutar k-means
  km_res <- stats::kmeans(diffs, centers = k, nstart = ns)
  
  # Crear objeto con clase personalizada
  classification_result <- data.frame(cd_result = cd_result, vl_result = vl_result, classification = as.factor(km_res$cluster))
  
  class(classification_result) <- c("InteractionClassification", "data.frame")
  
  return(classification_result)
}

#' Summarize Payoffs
#'
#' This function summarizes the payoffs object by classification.
#'
#' @param object A payoffs object.
#' @param ... Additional arguments (not used). 
#'
#' @export
#'
#' @examples
#' set.seed(42)
#' data(cd_3)
#' cd_data <- cd_3[,-1]
#' cd_result <- cds_diff(cd_data)
#' data(vl_3)
#' vl_data <- vl_3[,-1]
#' vl_result <- vlogs_diff(vl_data)
#' result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
#' data(preds)
#' payoffs_results <- estimate_payoffs(result, preds)
#' summary(payoffs_results)
summary <- function(object, ...) {
  UseMethod("summary")
}

#' Summarize Payoffs
#'
#' This function summarizes the payoffs object by classification.
#'
#' @param object A payoffs object.
#' @param ... Additional arguments (not used). 
#' @exportS3Method summary payoffs
summary.payoffs <- function(object, ...){
  group_means <- 
    magrittr::`%>%`(magrittr::`%>%`(object, dplyr::group_by(classification)),
                    dplyr::summarise(
                      cds_diff_mean = mean(cds3_diff), 
                      vlogs_diff_mean = mean(vlogs3_diff), 
                      payoff_mean = mean(predictions),
                      n = dplyr::n(),
                      .groups = 'drop'
                    )) 
  class(group_means) <- c("summary.payoffs", "data.frame")
  
  return(group_means)
}

utils::globalVariables(c("classification", "cds3_diff", "vlogs3_diff", "cds_diff_mean", "vlogs_diff_mean", "predictions", "payoff_mean"))

#' Compute Payoff Values for Quantum HIV Phenotype Interactions
#'
#' Computes payoff values for all pairwise combinations of quantum gate strategies provided in
#' a named list. For each pair, the function calculates the payoffs for both phenotypes `v` and `V`
#' using two different sets of payoff parameters.
#'
#' @param gates A named list of 2x2 unitary matrices representing quantum strategies (e.g., I, H, Z).
#' @param alpha Numeric scalar, payoff coefficient for phenotype `v` when both play `0`.
#' @param beta Numeric scalar, payoff coefficient for phenotype `v` when `v` plays `0`, `V` plays `1`.
#' @param gamma Numeric scalar, payoff coefficient for phenotype `v` when `v` plays `1`, `V` plays `0`.
#' @param theta Numeric scalar, payoff coefficient for phenotype `v` and `V` when both play `1`.
#' @param alpha2 Numeric scalar, alternate value of \code{alpha} for phenotype `v` in a second scenario.
#' @param beta2 Numeric scalar, alternate value of \code{beta} for phenotype `v` in a second scenario.
#' @param gamma2 Numeric scalar, alternate value of \code{gamma} for phenotype `v` in a second scenario.
#' @param theta2 Numeric scalar, alternate value of \code{theta} for phenotype `v` in a second scenario.
#'
#' @export
#'
#' @examples
#' I <- diag(2)
#' H <- 1 / sqrt(2) * matrix(c(1, 1, 1, -1), 2, 2)
#' Z <- diag(c(1, -1))
#' gates <- list(I = I, H = H, Z = Z)
#' payoffs <- payoffs_list(gates, 1, 0.5, 0.3,0.2, 1.5, 0.6, 0.7, 0.8)
payoffs_list <- function(gates, alpha, beta, gamma, theta, alpha2, beta2, gamma2, theta2) {
  # Ensure the gates input is a named list for easy access
  if (!is.list(gates) || is.null(names(gates))) {
    stop("Gates must be a named list of matrices.")
  }
  
  # Initialize an empty list to store the payoffs
  pays_list <- list()
  
  # Iterate over the gates and generate names dynamically
  for (gate1_name in names(gates)) {
    for (gate2_name in names(gates)) {
      
      g1 <- gates[[gate1_name]]
      g2 <- gates[[gate2_name]]
      
      # Compute for first set of payoffs
      phen <- phen_hiv(g1, g2, alpha, beta, gamma, theta)
      pays_list[[paste0("v_", gate1_name, "_", gate2_name)]] <- phen$pi_v
      pays_list[[paste0("V_", gate1_name, "_", gate2_name)]] <- phen$pi_V
      
      # Compute for second set of payoffs
      phen_b <- phen_hiv(g1, g2, alpha2, beta2, gamma2, theta2)
      pays_list[[paste0("v_", gate1_name, "_", gate2_name, "_b")]] <- phen_b$pi_v
      pays_list[[paste0("V_", gate1_name, "_", gate2_name, "_b")]] <- phen_b$pi_V
    }
  }
  
  class(pays_list) <- c("QuantumPayoffs", "list")
  
  return(pays_list)
  
}

test_that("`nearest_payoff()` works as expected", {
  I <- diag(2)
  H <- 1 / sqrt(2) * matrix(c(1, 1, 1, -1), 2, 2)
  Z <- diag(c(1, -1))
  gates <- list(I = I, H = H, Z = Z)
  alpha <- 0.3; beta <- 0.1; gamma <- 0.5; theta <- 0.2
  alpha2 <- 0.35; beta2 <- 0.15; gamma2 <- 0.6; theta2 <- 0.25
  pays <- payoffs_list(gates, alpha, beta, gamma, theta, alpha2, beta2, gamma2, theta2)
  expect_snapshot(nearest_payoff(-0.2, pays))
})

#' Create Mean Differences from Longitudinal Viral Load Data
#'
#' This function calculates the mean differences of viral loads across time for each individual in the dataset.
#'
#' @param vl_data A data frame of longitudinal viral load values per individual, where rows represent patients and columns represent sequential measurements across time (e.g., years or visits).
#'
#' @return An object of class `"Interaction"` with the following components:
#' \describe{
#'   \item{vl3_diff}{Mean differences of raw viral load values.}
#' }
#'
#' @export
#'
#' @examples
#' data(vl_3)
#' vl_data <- vl_3[,-1]
#' result <- vl_diff(vl_data)
vl_diff <- function(vl_data){
  # Validación básica
  if (!is.data.frame(vl_data)# || !is.data.frame(vl_data)
  ) {
    stop("Input must be data frames.")
  }
  
  # Diferencias carga viral
  vl3_diff <- 
    magrittr::`%>%`(magrittr::`%>%`(magrittr::`%>%`(dplyr::rowwise(vl_data), dplyr::mutate(vl_diff_values = mean(diff(stats::na.omit(dplyr::c_across(dplyr::everything())))))),
                                     dplyr::ungroup()),
                     dplyr::pull(vl_diff_values))
  
  vl3_diff <- as.data.frame(vl3_diff)    
  
  class(vl3_diff) <- c("Interaction", "data.frame")
  return(vl3_diff)
}

utils::globalVariables(c("vl_diff_values"))

#' BB84 Quantum Key Distribution (QKD) Simulation
#'
#' Simulates the BB84 protocol for quantum key distribution (QKD), illustrating 
#' the difference in the Quantum Bit Error Rate (QBER) between an ideal channel 
#' (no interference) and a channel under eavesdropping (Eve). The simulation 
#' models the encoding, transmission, and measurement of quantum bits (qubits) 
#' following the original Bennett–Brassard (1984) protocol.
#'
#' @param eavesdropping_active Logical. If \code{TRUE}, the simulation includes 
#' an eavesdropper (Eve) who performs a measure-and-resend attack, introducing 
#' errors. If \code{FALSE}, the simulation runs under ideal, interference-free 
#' conditions. Default is \code{FALSE}.
#' @param key_length Integer. Length of the quantum key sequence to be 
#' simulated. The default is 1000, which provides good statistical stability.
#' @param test_ratio Numeric. Proportion (between 0 and 1) of bits that Alice 
#' and Bob compare to detect eavesdropping during the verification phase. 
#' Default is 0.2.
#' 
#' @return
#' A list of class \code{"BB84Simulation"} containing:
#' \describe{
#'   \item{\code{QBER}}{The observed Quantum Bit Error Rate between Alice's and Bob's bits.}
#'   \item{\code{Sifted_Length}}{Number of bits retained after basis reconciliation.}
#'   \item{\code{Final_Key_Length}}{Number of bits remaining after error testing.}
#'   \item{\code{Eavesdropping}}{Logical flag indicating whether eavesdropping was active.}
#' }
#'
#' @details
#' The BB84 protocol proceeds through the following stages:
#' \enumerate{
#'   \item \strong{Preparation:} Alice generates random bits and bases and encodes photons accordingly.
#'   \item \strong{Transmission:} Photons are sent over a quantum channel.
#'   \item \strong{Eavesdropping (optional):} Eve intercepts each photon, measures it using a random basis,
#'         and resends a new photon, introducing potential errors.
#'   \item \strong{Measurement:} Bob measures the incoming photons using his own random bases.
#'   \item \strong{Sifting:} Alice and Bob retain only bits measured with matching bases.
#'   \item \strong{Error estimation:} A random subset of the sifted bits is compared to estimate QBER.
#' }
#'
#' If the measured QBER exceeds a security threshold (commonly 15–25%), it indicates
#' that eavesdropping has occurred and the key must be discarded. In the absence of
#' interference, the QBER should be close to zero.
#'
#' @examples
#' # Scenario 1: Perfect Channel (No Eavesdropping)
#' results_no_eve <- run_bb84_simulation(eavesdropping_active = FALSE)
#' cat("--- Scenario 1: No Eavesdropping ---\n")
#' cat(paste("Sifted Key Length:", results_no_eve$Sifted_Length, "\n"))
#' cat(paste("Final Key Length:", results_no_eve$Final_Key_Length, "\n"))
#' cat(paste("Quantum Bit Error Rate (QBER):", round(results_no_eve$QBER * 100, 2), "%\n"))
#' cat("Result: Secure key established successfully.\n")
#'
#' # Scenario 2: Measure-and-Resend Attack
#' results_with_eve <- run_bb84_simulation(eavesdropping_active = TRUE)
#' cat("\n--- Scenario 2: With Eavesdropping ---\n")
#' cat(paste("Sifted Key Length:", results_with_eve$Sifted_Length, "\n"))
#' cat(paste("Final Key Length:", results_with_eve$Final_Key_Length, "\n"))
#' cat(paste("Quantum Bit Error Rate (QBER):", round(results_with_eve$QBER * 100, 2), "%\n"))
#'
#' if (results_with_eve$QBER > 0.15) {
#'   cat("High QBER detected. Eavesdropping likely — key discarded.\n")
#' } else {
#'   cat("No eavesdropping detected (unlikely in theory).\n")
#' }
#'
#' @references
#' Bennett, C. H., & Brassard, G. (1984).
#' \emph{Quantum cryptography: Public key distribution and coin tossing.}
#' Proceedings of IEEE International Conference on Computers, Systems and Signal Processing, 175–179.
#'
#' Nielsen, M. A., & Chuang, I. L. (2010).
#' \emph{Quantum Computation and Quantum Information.} Cambridge University Press.
#'
#' Rieffel, E. G., & Polak, W. H. (2011).
#' \emph{Quantum Computing: A Gentle Introduction.} MIT Press.
#'
#' @export
run_bb84_simulation <- function(eavesdropping_active = FALSE,
                                key_length = 1000,
                                test_ratio = 0.2) {
  alice_encode <- function(bit, base) {
    list(encoded_bit = bit, encoding_base = base)
  }
  
  # --- Alice genera bits y bases ---
  alice_bits <- sample(0:1, key_length, replace = TRUE)
  alice_bases <- sample(0:1, key_length, replace = TRUE)
  photons_sent <- mapply(alice_encode, alice_bits, alice_bases, SIMPLIFY = FALSE)
  original_alice_bits <- alice_bits
  
  # --- Ataque de Eve ---
  if (eavesdropping_active) {
    eve_bases <- sample(0:1, key_length, replace = TRUE)
    for (i in seq_len(key_length)) {
      a_info <- photons_sent[[i]]
      eve_base <- eve_bases[i]
      eve_bit <- if (a_info$encoding_base == eve_base) a_info$encoded_bit else sample(0:1, 1)
      photons_sent[[i]] <- list(encoded_bit = eve_bit, encoding_base = eve_base)
    }
  }
  
  # --- Bob mide los fotones ---
  bob_bases <- sample(0:1, key_length, replace = TRUE)
  bob_bits <- rep(NA, key_length)
  
  for (i in seq_len(key_length)) {
    photon <- photons_sent[[i]]
    bob_base <- bob_bases[i]
    bob_bits[i] <- if (photon$encoding_base == bob_base) photon$encoded_bit else sample(0:1, 1)
  }
  
  # --- Fase de Tamizado ---
  matching <- which(alice_bases == bob_bases)
  raw_key_alice <- original_alice_bits[matching]
  raw_key_bob <- bob_bits[matching]
  
  sifted_length <- length(raw_key_alice)
  test_size <- floor(sifted_length * test_ratio)
  test_indices <- sample(seq_len(sifted_length), test_size)
  
  # --- Cálculo del QBER ---
  errors <- sum(raw_key_alice[test_indices] != raw_key_bob[test_indices])
  error_rate <- errors / test_size
  
  secret_indices <- setdiff(seq_len(sifted_length), test_indices)
  final_key_length <- length(secret_indices)
  
  # --- Resultado ---
  result <- list(
    QBER = error_rate,
    Sifted_Length = sifted_length,
    Final_Key_Length = final_key_length,
    Eavesdropping = eavesdropping_active
  )
  
  class(result) <- c("BB84Simulation", "list")
  return(result)
}

test_that("run_bb84_simulation() returns expected structure", {
  set.seed(42)
  result <- run_bb84_simulation(eavesdropping_active = FALSE, key_length = 500, test_ratio = 0.2)
  
  # Class and type checks
  testthat::expect_s3_class(result, "BB84Simulation")
  testthat::expect_type(result$QBER, "double")
  testthat::expect_type(result$Sifted_Length, "integer")
  testthat::expect_type(result$Final_Key_Length, "integer")
  testthat::expect_type(result$Eavesdropping, "logical")
  
  # Logical consistency checks
  testthat::expect_true(result$Sifted_Length <= 500)
  testthat::expect_true(result$Final_Key_Length <= result$Sifted_Length)
  testthat::expect_true(result$QBER >= 0)
  testthat::expect_true(result$QBER <= 1)
 })

#' Create Mean Standardized Differences from Longitudinal CD4 Data
#'
#' This function calculates the mean standardized differences of CD4 counts across time for each individual in the dataset.
#'
#' @param cd_data A data frame of longitudinal CD4 count values per individual, where rows represent patients and columns represent sequential measurements across time (e.g., years or visits).
#'
#' @return An object of class `"Interaction"` with the following components:
#' \describe{
#'   \item{cds3_diff}{Mean standardized differences of raw CD4 count values.}
#' }
#'
#' @export
#'
#' @examples
#' data(cd_3)
#' cd_data <- cd_3[,-1]
#' result <- cds_diff(cd_data)
cds_diff <- function(cd_data){
  # Validación básica
  if (!is.data.frame(cd_data)# || !is.data.frame(vl_data)
  ) {
    stop("Input must be data frames.")
  }
  cd_std <- dplyr::mutate(cd_data, dplyr::across(dplyr::everything(), ~ as.vector(scale(.))))
  cds3_diff <-  
    
    magrittr::`%>%`(magrittr::`%>%`(magrittr::`%>%`(dplyr::rowwise(cd_std), dplyr::mutate(cd_std_diff_values = mean(diff(stats::na.omit(dplyr::c_across(dplyr::everything())))))),
                                    dplyr::ungroup()),
                    dplyr::pull(cd_std_diff_values))
  
  cds3_diff <- as.data.frame(cds3_diff)
  class(cds3_diff) <- c("Interaction", "data.frame")
  return(cds3_diff)
}

utils::globalVariables(c("cd_std_diff_values"))

#' E91 Quantum Key Distribution (QKD) Simulation
#'
#' Simulates the E91 (Ekert 1991) quantum key distribution protocol using entangled
#' particle pairs (EPR) and Bell’s theorem (CHSH statistic) to ensure channel security.
#' Security is established when the \strong{Bell Inequality is violated}, i.e.
#' when \eqn{|S| > 2}, indicating quantum behavior.
#'
#' @param eavesdropping_active Logical. If \code{TRUE}, the entanglement is partially
#' destroyed, simulating an attack that forces the system toward the classical limit
#' where \eqn{|S| \leq 2}. Default is \code{FALSE}.
#' @param key_length Integer. Number of simulated EPR pairs (default = 1000).
#' @param noise_level Numeric. Noise level (0-1) applied when
#' \code{eavesdropping_active = TRUE}, reducing quantum correlation.
#' A value of 0.5 represents partial decoherence. Default is 0.5.
#'
#' @return A list of class \code{"E91Simulation"} containing:
#' \describe{
#'   \item{\code{S_Calculated}}{Observed Bell CHSH statistic.}
#'   \item{\code{S_Theoretical}}{Quantum theoretical value (\eqn{\approx} -2.8284).}
#'   \item{\code{Bell_Violation}}{\code{TRUE} if \eqn{|S| > 2} (secure),
#'   \code{FALSE} if \eqn{|S| \leq 2} (insecure).}
#'   \item{\code{Sifted_Key_Length}}{Number of bits retained for key formation.}
#'   \item{\code{Eavesdropping}}{Indicates if an attack was simulated.}
#' }
#'
#' @details
#' El E91 se diferencia de BB84 en que la detección del espía es \strong{simultánea} a la generación
#' de la clave. Los pares de bases son separados en dos conjuntos:
#' \enumerate{
#'  \item \strong{Clave Secreta:} Pares con perfecta anti-correlación (ej. a2-b1, a3-b2).
#'  \item \strong{Test de Bell:} Pares restantes usados para calcular la esperanza $E(a_i, b_j)$ y el valor $S$.
#' }
#' Si $S$ cae por debajo de 2 (límite clásico de Bell), se concluye que el entrelazamiento ha sido
#' roto por Eve, y la clave debe descartarse.
#'
#' @examples
#' # Escenario 1: Canal Cuántico Seguro (No Eavesdropping)
#' results_secure <- run_e91_simulation(eavesdropping_active = FALSE)
#' cat("--- Escenario 1: Sin Eavesdropping ---\n")
#' cat(paste("S Calculado:", round(results_secure$S_Calculated, 4), "\n"))
#' cat(paste("Violación de Bell:", results_secure$Bell_Violation, "\n"))
#'
#' # Escenario 2: Ataque que Rompe el Entrelazamiento
#' results_attack <- run_e91_simulation(eavesdropping_active = TRUE)
#' cat("\n--- Escenario 2: Con Ataque ---\n")
#' cat(paste("S Calculado:", round(results_attack$S_Calculated, 4), "\n"))
#' cat(paste("Violación de Bell:", results_attack$Bell_Violation, "\n"))
#'
#' @references
#' Ekert, A. K. (1991). \emph{Quantum cryptography based on Bell's theorem.}
#' Physical Review Letters, 67(6), 661.
#'
#' @export
run_e91_simulation <- function(eavesdropping_active = FALSE,
                               key_length = 1000,
                               noise_level = 0.5) {
  
  # --- 1. Definición de Parámetros y Bases ---
  ALICE_BASES_ANGLES <- c(0, pi/4, pi/2)
  names(ALICE_BASES_ANGLES) <- c("a1", "a2", "a3")
  BOB_BASES_ANGLES <- c(pi/4, pi/2, 3*pi/4)
  names(BOB_BASES_ANGLES) <- c("b1", "b2", "b3")
  
  S_QUANTUM_PREDICTION <- -2 * sqrt(2) 
  S_CLASSIC_LIMIT <- 2.0
  
  quantum_correlation_E <- function(theta_a, theta_b) {
    return(-cos(theta_a - theta_b))
  }
  
  simulate_entangled_measurement <- function(alice_angle, bob_angle, eavesdropping) {
    E_ideal <- quantum_correlation_E(alice_angle, bob_angle)
    
    # Simula ataque: reduce correlación hacia 0 (comportamiento clásico).
    E_actual <- if (eavesdropping) E_ideal * (1 - noise_level) else E_ideal
    
    P_same_sign <- (1 + E_actual) / 2 
    alice_result <- sample(c(1, -1), 1)
    
    bob_result <- if (stats::runif(1) < P_same_sign) alice_result else -alice_result 
    
    return(c(alice = alice_result, bob = bob_result))
  }
  
  # --- 2. Generación de Datos y Mediciones ---
  
  alice_base_indices <- sample(1:3, key_length, replace = TRUE)
  bob_base_indices <- sample(1:3, key_length, replace = TRUE)
  results_matrix <- matrix(NA, nrow = key_length, ncol = 2)
  key_bases <- matrix(NA, nrow = key_length, ncol = 2)
  
  for (i in 1:key_length) {
    a_idx <- alice_base_indices[i]
    b_idx <- bob_base_indices[i]
    theta_a <- ALICE_BASES_ANGLES[a_idx]
    theta_b <- BOB_BASES_ANGLES[b_idx]
    
    measurement <- simulate_entangled_measurement(theta_a, theta_b, eavesdropping_active)
    
    results_matrix[i, ] <- measurement
    key_bases[i, ] <- c(names(ALICE_BASES_ANGLES)[a_idx], names(BOB_BASES_ANGLES)[b_idx])
  }
  
  alice_results <- results_matrix[, 1]
  bob_results <- results_matrix[, 2]
  
  # --- 3. Tamizado (Sifting) para la Clave ---
  
  # Pares de bases que forman la clave (anti-correlación perfecta)
  key_indices <- which(
    (key_bases[, 1] == "a2" & key_bases[, 2] == "b1") | 
      (key_bases[, 1] == "a3" & key_bases[, 2] == "b2")    
  )
  
  raw_key_alice <- alice_results[key_indices]
  raw_key_bob_raw <- bob_results[key_indices]
  
  # Bob debe FLIPPEAR su resultado (-1 a +1) debido a la anti-correlación del estado singlet
  raw_key_bob_final <- -raw_key_bob_raw 
  key_length_sifted <- length(raw_key_alice)
  
  # --- 4. Test de Bell (Cálculo de S) ---
  
  calculate_E <- function(a_name, b_name, key_bases, alice_results, bob_results) {
    indices <- which(key_bases[, 1] == a_name & key_bases[, 2] == b_name)
    if (length(indices) < 2) return(NA)
    
    # E = Promedio del producto de los resultados (R_Alice * R_Bob)
    E_value <- mean(alice_results[indices] * bob_results[indices])
    return(E_value)
  }
  
  # Fórmula CHSH: S = E(a1, b1) - E(a1, b3) + E(a3, b1) + E(a3, b3)
  E_a1_b1 <- calculate_E("a1", "b1", key_bases, alice_results, bob_results)
  E_a1_b3 <- calculate_E("a1", "b3", key_bases, alice_results, bob_results)
  E_a3_b1 <- calculate_E("a3", "b1", key_bases, alice_results, bob_results)
  E_a3_b3 <- calculate_E("a3", "b3", key_bases, alice_results, bob_results)
  
  S_calculated <- E_a1_b1 - E_a1_b3 + E_a3_b1 + E_a3_b3
  
  # --- 5. Resultado ---
  
  bell_violation <- abs(S_calculated) > S_CLASSIC_LIMIT
  
  result <- list(
    S_Calculated = S_calculated,
    S_Theoretical = S_QUANTUM_PREDICTION,
    S_Classic_Limit = S_CLASSIC_LIMIT,
    Bell_Violation = bell_violation,
    Sifted_Key_Length = key_length_sifted,
    Eavesdropping = eavesdropping_active
  )
  
  class(result) <- c("E91Simulation", "list")
  
  return(result)
}

test_that("run_e91_simulation() returns expected structure", {
  set.seed(42)
  # Usaremos un key_length bajo para que las pruebas de valores específicos sean más estables
  result <- run_e91_simulation(eavesdropping_active = FALSE, key_length = 2000, noise_level = 0)
  
  # --- 1. Class and Type Checks ---
  testthat::expect_s3_class(result, "E91Simulation")
  testthat::expect_type(result$S_Calculated, "double")
  testthat::expect_type(result$S_Theoretical, "double")
  testthat::expect_type(result$S_Classic_Limit, "double")
  testthat::expect_type(result$Bell_Violation, "logical")
  testthat::expect_type(result$Sifted_Key_Length, "integer")
  testthat::expect_type(result$Eavesdropping, "logical")
})

#' Find Nearest Payoff
#'
#' This function computes the nearest simulated payoff from a given list of payoffs 
#' based on a viral load difference (vl_diff). It returns both the nearest payoff value 
#' and its corresponding payoff name.
#'
#' @param vl_diff Numeric value representing the viral load difference for which the 
#' nearest payoff will be found.
#' @param payoffs_list A named list of payoff values, where the names correspond to 
#' specific payoffs and the values are the associated payoff values. 
#'
#' @export
#'
#' @examples
#'  I <- diag(2)
#'  H <- 1 / sqrt(2) * matrix(c(1, 1, 1, -1), 2, 2)
#'  Z <- diag(c(1, -1))
#'  gates <- list(I = I, H = H, Z = Z)
#'  alpha <- 0.3; beta <- 0.1; gamma <- 0.5; theta <- 0.2
#'  alpha2 <- 0.35; beta2 <- 0.15; gamma2 <- 0.6; theta2 <- 0.25
#'  pays <- payoffs_list(gates, alpha, beta, gamma, theta, alpha2, beta2, gamma2, theta2)
#'  nearest_payoff(-0.2, pays)
nearest_payoff <- function(vl_diff, payoffs_list){
  stopifnot(is.numeric(vl_diff), length(vl_diff) == 1)
  stopifnot(is.list(payoffs_list), !is.null(names(payoffs_list)))
  payoff_values <- unlist(payoffs_list)
  payoff_names <- names(payoffs_list)
  differences <- abs(vl_diff - payoff_values)
  min_index <- which.min(differences)
  pays_list = list(value = payoff_values[min_index], name = payoff_names[min_index])
  class(pays_list) <- c("NearestPayoff", "list")
  return(pays_list)
}

#' Calculate Final State and Payoffs in Quantum Game
#'
#' This function calculates the final quantum state and expected payoffs for two 
#' players in a quantum game based on their strategies. The function uses quantum 
#' gates and unitary transformations to simulate the game dynamics.
#'
#' @param strategy1 A 2x2 matrix representing the strategy of player 1.
#' @param strategy2 A 2x2 matrix representing the strategy of player 2.
#' @param alpha A numeric value representing the payoff for outcome |00>.
#' @param beta A numeric value representing the payoff for outcome |01>.
#' @param gamma A numeric value representing the payoff for outcome |10>.
#' @param theta A numeric value representing the payoff for outcome |11>. 
#'
#' @export
#'
#' @examples
#' strategy1 <- diag(2) # Identity matrix for strategy 1
#' strategy2 <- diag(2) # Identity matrix for strategy 2
#' alpha <- 1
#' beta <- 0.5
#' gamma <- 2
#' theta <- 0.1
#' result <- phen_hiv(strategy1, strategy2, alpha, beta, gamma, theta)
#' 
#' @references
#' Özlüer Başer, B. (2022). "Analyzing the competition of HIV-1 phenotypes with quantum game theory". 
#' Gazi University Journal of Science, 35(3), 1190--1198. \doi{10.35378/gujs.772616}
phen_hiv <- function(strategy1, strategy2, alpha, beta, gamma, theta){
  # Define the quantum gates
  I <- diag(2)
  X <- matrix(c(0, 1, 1, 0), nrow=2)
  
  # Define the initial state |00>
  initial_state <- array(complex(real = c(1,0,0,0), imaginary = c(0,0,0,0)), dim = c(4, 1))
  
  # Define the unitary transformation for mutant H gate
  U <- 1/sqrt(2) * (kronecker(I, I) - 1i * kronecker(X, X))
  V <- 1/sqrt(2) * (kronecker(I, I) + 1i * kronecker(X, X))
  
  # Apply the sequence of operations to the initial state
  state_after_U <- U %*% initial_state
  state_after_strategy <- kronecker(strategy1, strategy2) %*% state_after_U
  state_after_V <- V %*% state_after_strategy
  final_state <- U %*% state_after_V
  
  # Apply the inverse unitary transformation
  U_inverse <- Conj(t(U))
  psi_f <- U_inverse %*% final_state
  
  # Calculate the probabilities for each basis state
  prob_00 <- Mod(psi_f[1])^2
  prob_01 <- Mod(psi_f[2])^2
  prob_10 <- Mod(psi_f[3])^2
  prob_11 <- Mod(psi_f[4])^2
  
  # Calculate the expected payoffs for players v and V
  pi_v <- alpha * prob_00 + beta * prob_01 + gamma * prob_10 + theta * prob_11
  pi_V <- alpha * prob_00 + gamma * prob_01 + beta * prob_10 + theta * prob_11
  
  phen <- data.frame(prob_00 = prob_00, prob_01 = prob_01, prob_10 = prob_10, prob_11 = prob_11,
             pi_v = pi_v, pi_V = pi_V)
  
  class(phen) <- c("qgame", "data.frame")
  
  return(phen)  
  
}

test_that("phen_hiv() works as expected", {
  strategy1 <- diag(2) 
  strategy2 <- diag(2) 
  alpha <- 1
  beta <- 0.5
  gamma <- 2
  theta <- 0.1 
  expect_snapshot(
    phen_hiv(strategy1, strategy2, alpha, beta, gamma, theta) 
  )
})

#' Summarize an InteractionClassification object
#'
#' Computes summary statistics by classification group from an object of class
#' `InteractionClassification`, including mean differences in viral load and CD4 counts,
#' and the number of observations per cluster.
#'
#' @param object An object of class `InteractionClassification` returned by the
#'   [InteractionClassification()] function.
#' @param ... Additional arguments passed to other methods (currently not used).   
#'
#' @returns A `data.frame` with one row per interaction cluster and the following columns:
#' \describe{
#'   \item{classification}{Cluster label (as factor).}
#'   \item{cds_diff_mean}{Mean of CD4 differences in the group.}
#'   \item{vlogs_diff_mean}{Mean of viral load differences in the group.}
#'   \item{n}{Number of observations in the group.}
#' }
#'
#' @export
#'
#' @examples
#' set.seed(42)
#' data(cd_3)
#' cd_data <- cd_3[,-1]
#' cd_result <- cds_diff(cd_data)
#' data(vl_3)
#' vl_data <- vl_3[,-1]
#' vl_result <- vlogs_diff(vl_data)
#' result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
#' summary(result)
summary <- function(object, ...) {
  UseMethod("summary")
}

#' Summarize an InteractionClassification object
#'
#' Computes summary statistics by classification group from an object of class
#' `InteractionClassification`, including mean differences in viral load and CD4 counts,
#' and the number of observations per cluster.
#'
#' @param object An object of class `InteractionClassification` returned by the
#'   [InteractionClassification()] function.
#' @param ... Additional arguments passed to other methods (currently not used). 
#' @exportS3Method summary InteractionClassification
summary.InteractionClassification <- function(object, ...) {
  group_means <- 
    magrittr::`%>%`(magrittr::`%>%`(object, dplyr::group_by(classification)),
                    dplyr::summarise(
                      cds_diff_mean = mean(cds3_diff), 
                      vlogs_diff_mean = mean(vlogs3_diff), 
                      n = dplyr::n(),
                      .groups = 'drop'
                    )) 
  class(group_means) <- c("summary.InteractionClassification", "data.frame")
  
  return(group_means)
}

utils::globalVariables(c("classification", "cds3_diff", "vlogs3_diff", "cds_diff_mean", "vlogs_diff_mean"))

#' Longitudinal CD4 Lymphocyte Counts for HIV Patients (2018-2024)
#'
#' @description Contains longitudinal measurements of CD4 lymphocyte counts for 
#' 176 patients living with HIV, recorded over the period from 2018 to 2024. 
#' CD4 counts are a critical indicator of immune function, used to monitor the 
#' progression of HIV and the effectiveness of treatments. Measurements were 
#' taken at various points throughout the study, with some missing values due 
#' to unavailable data for specific patients at certain times.
#'
#' @format A data frame with 176 rows and 18 variables:
#' \describe{
#'   \item{ID}{Unique identifier for each patient.}
#'   \item{cd_2018_1}{CD4 count for the first measurement in 2018.}
#'   \item{cd_2018_2}{CD4 count for the second measurement in 2018.}
#'   \item{cd_2019_1}{CD4 count for the first measurement in 2019.}
#'   \item{cd_2019_2}{CD4 count for the second measurement in 2019.}
#'   \item{cd_2020_1}{CD4 count for the first measurement in 2020.}
#'   \item{cd_2021_1}{CD4 count for the first measurement in 2021.}
#'   \item{cd_2021_2}{CD4 count for the second measurement in 2021.}
#'   \item{cd_2021_3}{CD4 count for the third measurement in 2021.}
#'   \item{cd_2022_1}{CD4 count for the first measurement in 2022.}
#'   \item{cd_2022_2}{CD4 count for the second measurement in 2022.}
#'   \item{cd_2022_3}{CD4 count for the third measurement in 2022.}
#'   \item{cd_2023_1}{CD4 count for the first measurement in 2023.}
#'   \item{cd_2023_2}{CD4 count for the second measurement in 2023.}
#'   \item{cd_2023_3}{CD4 count for the third measurement in 2023.}
#'   \item{cd_2024_1}{CD4 count for the first measurement in 2024.}
#'   \item{cd_2024_2}{CD4 count for the second measurement in 2024.}
#'   \item{cd_2024_3}{CD4 count for the third measurement in 2024.}
#' }
#'
#' @details .
#' CD4 counts are used to monitor immune system health in individuals with 
#' HIV. A lower CD4 count often indicates a weakened immune system, whereas 
#' higher counts suggest a stronger immune response. Some values are missing, 
#' indicating no measurement was taken for a particular patient at that time.
#'
#' @examples
#' # Load the dataset
#' data(cd_3)
#'
#' # Summarize CD4 counts for the year 2021
#' summary(cd_3[, c("cd_2021_1", "cd_2021_2", "cd_2021_3")])
#'
#' @source Clinical data from Hospital Vicente Guerrero, IMSS, HIV Clinic.
"cd_3"


#' Longitudinal Viral Load Values for HIV Patients (2018-2024)
#'
#' @description Contains longitudinal measurements of viral load for 176 
#' patients from 2018 to 2024. Viral load is a critical marker used to monitor 
#' the effectiveness of HIV treatment by measuring the amount of HIV RNA in the 
#' blood.
#'
#' @format A data frame with 176 rows and 18 variables:
#' \describe{
#'   \item{ID}{Unique identifier for each patient.}
#'   \item{vl_2018_1}{Viral load for the first measurement in 2018.}
#'   \item{vl_2018_2}{Viral load for the second measurement in 2018.}
#'   \item{vl_2019_1}{Viral load for the first measurement in 2019.}
#'   \item{vl_2019_2}{Viral load for the second measurement in 2019.}
#'   \item{vl_2020_1}{Viral load for the first measurement in 2020.}
#'   \item{vl_2021_1}{Viral load for the first measurement in 2021.}
#'   \item{vl_2021_2}{Viral load for the second measurement in 2021.}
#'   \item{vl_2021_3}{Viral load for the third measurement in 2021.}
#'   \item{vl_2022_1}{Viral load for the first measurement in 2022.}
#'   \item{vl_2022_2}{Viral load for the second measurement in 2022.}
#'   \item{vl_2022_3}{Viral load for the third measurement in 2022.}
#'   \item{vl_2023_1}{Viral load for the first measurement in 2023.}
#'   \item{vl_2023_2}{Viral load for the second measurement in 2023.}
#'   \item{vl_2023_3}{Viral load for the third measurement in 2023.}
#'   \item{vl_2024_1}{Viral load for the first measurement in 2024.}
#'   \item{vl_2024_2}{Viral load for the second measurement in 2024.}
#'   \item{vl_2024_3}{Viral load for the third measurement in 2024.}
#' }
#'
#' @details
#' The viral load measurements provide insight into the patient's response to 
#' antiretroviral therapy (ART). Lower viral load values, especially 
#' undetectable levels, indicate better control of the infection. Missing 
#' values indicate that no viral load measurement was available for that patient 
#' at that specific time.
#'
#' @examples
#' \dontrun{
#' # Load the dataset
#' data(vl_3)
#'
#' # Summarize viral loads for the year 2021
#' summary(vl_3[, c("cd_2021_1", "cd_2021_2", "cd_2021_3")])
#' }
#'
#' @source Clinical data from Hospital Vicente Guerrero, IMSS, HIV Clinic.
"vl_3"


#' Quantum Phenotype Interactions in HIV Model
#'
#' The `qphen` dataset contains 176 observations and 24 variables, representing classified phenotype interactions in a quantum game-theoretic model of HIV phenotypes.
#' The data includes CD4 and viral load differences, quantum game strategies, classification clusters, and tuberculosis/genoresistance indicators.
#'
#' @format A data frame with 176 rows and 24 variables:
#' \describe{
#'   \item{id}{(double) Unique identifier for each observation.}
#'   \item{vl_diff}{(double) Difference in viral load (log scale) between time points.}
#'   \item{cd_diff}{(double) Difference in CD4 count between time points.}
#'   \item{vlogs_diff_mean}{(double) Mean difference of viral loads across the dataset.}
#'   \item{cds_diff_mean}{(double) Mean difference of CD4 counts across the dataset.}
#'   \item{n}{(double) Number of cases in each interaction cluster.}
#'   \item{payoffs}{(double) Computed payoff for the phenotype interaction.}
#'   #'   \item{payoffs_b}{(double) Alternative computed payoff.}
#'   \item{nearest_payoff}{(double) Closest estimated payoff value.}
#'   \item{classification_2}{(double) Cluster assignment for phenotype interactions (second clustering method).}
#'   \item{classification_3}{(double) Cluster assignment for phenotype interactions (third clustering method).}
#'   \item{classification_4}{(double) Cluster assignment for phenotype interactions (fourth clustering method).}
#'   \item{phen_1}{(double) Phenotype type (`v` or `V`).}
#'   \item{str1_2}{(double) Strategy of the first phenotype using `X`, `T`, or `H` gate (binary encoding).}
#'   \item{str1_3}{(double) Alternative strategy of the first phenotype.}
#'   \item{str2_2}{(double) Strategy of the second phenotype using `H`, `Id`, `S`, `T`, `X`, `Y`, or `Z` gate (binary encoding).}
#'   \item{str2_3}{(double) Alternative strategy of the second phenotype.}
#'   \item{str2_4}{(double) Alternative strategy of the second phenotype.}
#'   \item{str2_5}{(double) Alternative strategy of the second phenotype.}
#'   \item{str2_6}{(double) Alternative strategy of the second phenotype.}
#'   \item{str2_7}{(double) Alternative strategy of the second phenotype.}
#'   \item{batch_1}{(double) Indicates whether predictions were made on full data or batch data.}
#'   \item{TB_1}{(double) Indicator for tuberculosis presence (1 = TB, 0 = no TB).}
#'   \item{GR_1}{(double) Indicator for genoresistance presence (1 = resistant, 0 = non-resistant).}
#' }
#'
#' @usage data(qphen)
#'
#' @keywords datasets
#'
#' @examples
#' data(qphen)
#' head(qphen)
"qphen"


#' Predictions for Longitudinal Viral Load Values for HIV Patients (2018-2024)
#'
#' @description Contains predictions of longitudinal viral load values for 176 
#' patients from 2018 to 2024. 
#'
#' @examples
#' data(preds)
#' head(preds)
#'
#' @source Clinical data from Hospital Vicente Guerrero, IMSS, HIV Clinic.
"preds"


#' Batched Predictions for Longitudinal Viral Load Values for HIV Patients (2018-2024)
#'
#' @description Contains batched predictions of longitudinal viral load values for 176 
#' patients from 2018 to 2024. 
#'
#' @examples
#' data(preds2)
#' head(preds2)
#'
#' @source Clinical data from Hospital Vicente Guerrero, IMSS, HIV Clinic.
"preds2"


#' Angle-Encoding Dataset for 3-Qubit Quantum State Preparation
#'
#' @description A numeric data frame containing patient-level biomarker expression values
#' used for angle encoding into a 3-qubit quantum state. Each row corresponds
#' to a sample (patient), and each column corresponds to a biomarker mapped
#' to a rotation angle in the quantum circuit.
#'
#' @details
#' The encoding follows a 3-qubit angle encoding scheme:
#'
#' \deqn{
#' |\psi_j\rangle = \bigotimes_{i=1}^3 R_y(\theta_{j,i}) R_z(\phi_{j,i}) |0\rangle
#' }
#'
#' where each qubit is parameterized by two biomarkers:
#'
#' \itemize{
#'   \item \strong{Qubit 1:} \code{IL6} (\eqn{\theta_1}), \code{IDO1} (\eqn{\phi_1})
#'   \item \strong{Qubit 2:} \code{TDO2} (\eqn{\theta_2}), \code{CD14} (\eqn{\phi_2})
#'   \item \strong{Qubit 3:} \code{HADH} (\eqn{\theta_3}), \code{LDHB} (\eqn{\phi_3})
#' }
#'
#' These biomarkers capture key immunometabolic axes, including inflammation,
#' tryptophan metabolism, innate immune sensing, and metabolic reprogramming.
#'
#' Prior to encoding, values are typically normalized to angular ranges
#' (e.g., \eqn{[0, 2\pi]}) using dataset-specific min–max scaling.
#'
#' @format A data frame with 12 rows and 6 variables:
#' \describe{
#'   \item{CD14}{Expression level of CD14 (innate immune receptor; phi2 angle).}
#'   \item{HADH}{Expression level of HADH (beta-oxidation marker; theta3 angle).}
#'   \item{IDO1}{Expression level of IDO1 (tryptophan metabolism; phi1 angle).}
#'   \item{IL6}{Expression level of IL6 (inflammatory signaling; theta1 angle).}
#'   \item{LDHB}{Expression level of LDHB (glycolytic flux; phi3 angle).}
#'   \item{TDO2}{Expression level of TDO2 (systemic metabolic repression; theta2 angle).}
#' }
#'
#' @usage data(data_enc)
#'
#' @examples
#' data(data_enc)
#' str(data_enc)
#'
#' @references
#' Brown, A. J., et al. (2024). Metabolic reprogramming of human macrophages during Mycobacterium tuberculosis infection under HIV exposure. Gene Expression Omnibus (GEO), accession GSE314344.
#' Baluku, J. B., et al. (2023). DNA methylation profiling of people living with HIV stratified by tuberculosis status. Gene Expression Omnibus (GEO), accession GSE304107.
#'
"data_enc"


#' Biomarker Correlation Dataset for Hamiltonian Construction
#'
#' @description A numeric data frame containing a curated panel of biomarkers used to
#' estimate correlation-driven coefficients for a variational Hamiltonian
#' in a 3-qubit quantum model.
#'
#' @details
#' The Hamiltonian is defined as:
#'
#' \deqn{
#' \hat{H} = \sum_{k=1}^K c_k P_k
#' }
#'
#' where:
#' \itemize{
#'   \item \eqn{P_k} are Pauli strings from the 3-qubit Pauli group
#'   \item \eqn{c_k} are coefficients estimated via Pearson correlation
#'   between selected biomarkers
#' }
#'
#' The biomarkers in \code{data_corr} are grouped into biological axes:
#'
#' \itemize{
#'   \item \strong{Amino Acid Metabolism:} IDO1, TDO2, ACMSD, CPS1, FAH
#'   \item \strong{Immune Signaling:} IL6, CCL4, CCL5, CCL13, NT5E
#'   \item \strong{Antigen Processing:} CD14, CD163, LAT, CTSL, CTSD
#'   \item \strong{Fatty Acid / Lipid:} HADH, FASN, LTC4S, PTGS2, PTGES
#'   \item \strong{Aerobic / Nucleotide:} LDHB, PKM, IMPDH2, ADA, ADK, HK2
#'   \item \strong{Regulatory / Epigenetic:} ZNF93, MTF1, RPTOR, ZNF708, CAD
#' }
#'
#' These features define a high-dimensional correlation structure used to
#' construct patient-specific or cohort-level Hamiltonians.
#'
#' @format A data frame with 12 rows and 30 variables:
#' \describe{
#'   \item{ACMSD}{Amino acid metabolism biomarker.}
#'   \item{ADA}{Nucleotide metabolism enzyme.}
#'   \item{ADK}{Adenosine kinase (nucleotide metabolism).}
#'   \item{CCL13}{Chemokine involved in immune signaling.}
#'   \item{CCL4}{Inflammatory chemokine.}
#'   \item{CCL5}{Immune signaling chemokine.}
#'   \item{CD14}{Innate immune receptor.}
#'   \item{CD163}{Macrophage activation marker.}
#'   \item{CPS1}{Urea cycle enzyme.}
#'   \item{CTSD}{Lysosomal protease.}
#'   \item{CTSL}{Cathepsin L (antigen processing).}
#'   \item{FAH}{Fumarylacetoacetate hydrolase.}
#'   \item{FASN}{Fatty acid synthase.}
#'   \item{HADH}{Beta-oxidation enzyme.}
#'   \item{HK2}{Hexokinase 2 (glycolysis).}
#'   \item{IDO1}{Tryptophan metabolism enzyme.}
#'   \item{IL6}{Inflammatory cytokine.}
#'   \item{IMPDH2}{Purine biosynthesis enzyme.}
#'   \item{LAT}{T-cell activation linker protein.}
#'   \item{LDHB}{Lactate dehydrogenase B.}
#'   \item{LTC4S}{Leukotriene synthesis enzyme.}
#'   \item{MTF1}{Metal regulatory transcription factor.}
#'   \item{NT5E}{Ecto-5'-nucleotidase.}
#'   \item{PKM}{Pyruvate kinase (glycolysis).}
#'   \item{PTGES}{Prostaglandin synthesis enzyme.}
#'   \item{PTGS2}{Cyclooxygenase-2 (inflammation).}
#'   \item{RPTOR}{mTOR pathway regulator.}
#'   \item{TDO2}{Tryptophan metabolism enzyme.}
#'   \item{ZNF708}{Zinc finger transcription factor.}
#'   \item{ZNF93}{Zinc finger regulatory protein.}
#' }
#'
#' @usage data(data_corr)
#'
#' @examples
#' data(data_corr)
#' dim(data_corr)
#' 
#' @references
#' Brown, A. J., et al. (2024). Metabolic reprogramming of human macrophages during Mycobacterium tuberculosis infection under HIV exposure. Gene Expression Omnibus (GEO), accession GSE314344.
#' Baluku, J. B., et al. (2023). DNA methylation profiling of people living with HIV stratified by tuberculosis status. Gene Expression Omnibus (GEO), accession GSE304107.
#'
"data_corr"

#' Mean Squared Errors for Interaction Classification
#'
#' Mean squared errors (MSE) for viral load differences
#' and CD4 count differences by comparing the actual values with the group means 
#' from the classification.
#'
#' @param object An object of class \code{InteractionClassification} containing 
#'               the classified data and clustering results.
#' @param ... Additional arguments passed to other methods (currently not used).
#' 
#' @return A `data.frame` containing the MSE for CD4 count differences (\code{mse_cds_diff})
#'         and (\code{mse_vlogs_diff}) for viral load differences.
#' @export
#'
#' @examples
#' set.seed(42)
#' data(cd_3)
#' cd_data <- cd_3[,-1]
#' cd_result <- cds_diff(cd_data)
#' data(vl_3)
#' vl_data <- vl_3[,-1]
#' vl_result <- vlogs_diff(vl_data)
#' result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
#' mse(result)
#' 
#' @export
mse <- function(object, ...) {
  UseMethod("mse")
}

#' Mean Squared Errors for Interaction Classification
#'
#' Mean squared errors (MSE) for viral load differences
#' and CD4 count differences by comparing the actual values with the group means 
#' from the classification.
#'
#' @param object An object of class \code{InteractionClassification} containing 
#'               the classified data and clustering results.
#' @param ... Additional arguments passed to other methods (currently not used).
#' @exportS3Method mse InteractionClassification
mse.InteractionClassification <- function(object, ...){
  group_means <- summary.InteractionClassification(object)
  
  merged_result <-  
    magrittr::`%>%`(magrittr::`%>%`(magrittr::`%>%`(object, dplyr::left_join(group_means, by = "classification")),
                                    dplyr::mutate(
                                      cds_diff_e = cds3_diff - cds_diff_mean,
                                      vlogs_diff_e = vlogs3_diff - vlogs_diff_mean, 
                                      cds_diff_se = (cds_diff_e)^2,
                                      vlogs_diff_se = (vlogs_diff_e)^2
                                    )
    ),
    dplyr::summarise(
      mse_cds_diff = mean(cds_diff_se), 
      mse_vlogs_diff = mean(vlogs_diff_se), 
      n = dplyr::n(),
      .groups = 'drop'
    ))
  class(merged_result) <- c("mse.InteractionClassification", "data.frame")
  
  return(merged_result)  
}

utils::globalVariables(c("cds3_diff", "cds_diff_mean", "cds_diff_e", "cds_diff_se", "vlogs3_diff", "vlogs_diff_mean", "vlogs_diff_e", "vlogs_diff_se"))

test_that("`mse.InteractionClassification()` works as expected", {
  set.seed(42)
  data(cd_3)
  cd_data <- cd_3[,-1]
  cd_result <- cds_diff(cd_data)
  data(vl_3)
  vl_data <- vl_3[,-1]
  vl_result <- vlogs_diff(vl_data)
  result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
  expect_snapshot({
    mse(result)
  })
})

#' Estimate Payoff Parameters for HIV Phenotype Interactions
#'
#' This function estimates the payoff parameters for HIV phenotype interactions
#' based on the provided classification object and predictions from a viral
#' load model. It calculates the mean differences in viral loads and CD4 counts,
#' as well as the average payoffs for each classification.
#'
#' @param object An object of class `InteractionClassification` containing
#' the data on viral load differences and CD4 counts.
#' @param predictions A `data.frame` containing predictions of viral loads or CD4 values.
#'
#' @export
#'
#' @examples
#' set.seed(42)
#' data(cd_3)
#' cd_data <- cd_3[,-1]
#' cd_result <- cds_diff(cd_data)
#' data(vl_3)
#' vl_data <- vl_3[,-1]
#' vl_result <- vlogs_diff(vl_data)
#' result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
#' data(preds)
#' payoffs_results <- estimate_payoffs(result, preds)
estimate_payoffs <- function(object, predictions){
  group_means <- summary.InteractionClassification(object)
  
  merged_result <-  
    magrittr::`%>%`(magrittr::`%>%`(magrittr::`%>%`(object, dplyr::left_join(group_means, by = "classification")),
                                    dplyr::mutate(
                                      cds_diff_e = cds3_diff - cds_diff_mean,
                                      vlogs_diff_e = vlogs3_diff - vlogs_diff_mean, 
                                      cds_diff_se = (cds_diff_e)^2,
                                      vlogs_diff_se = (vlogs_diff_e)^2
                                    )), 
    dplyr::bind_cols(payoffs = predictions))
  class(merged_result) <- c("payoffs", "data.frame")
  
  return(merged_result)  
  
}

utils::globalVariables(c("cds3_diff", "cds_diff_mean", "cds_diff_e", "cds_diff_se", "vlogs3_diff", "vlogs_diff_mean", "vlogs_diff_e", "vlogs_diff_se"))

#' Mean Squared Errors for Payoff Predictions
#'
#' Computes the mean squared error (MSE) between observed CD4 and viral load differences 
#' and their corresponding predicted payoff values within each interaction classification.
#'
#' @param object An object of class `payoffs`.
#' @param ... Additional arguments passed to other methods (currently not used).
#'
#' @examples
#' set.seed(42)
#' data(cd_3)
#' cd_data <- cd_3[,-1]
#' cd_result <- cds_diff(cd_data)
#' data(vl_3)
#' vl_data <- vl_3[,-1]
#' vl_result <- vlogs_diff(vl_data)
#' result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
#' data(preds)
#' payoffs_results <- estimate_payoffs(result, preds)
#' mse(payoffs_results)
#' 
#' @export
mse <- function(object, ...) {
  UseMethod("mse")
}

#' Mean Squared Errors for Payoff Predictions
#'
#' Computes the mean squared error (MSE) between observed CD4 and viral load differences 
#' and their corresponding predicted payoff values within each interaction classification.
#'
#' @param object An object of class `payoffs`.
#' @param ... Additional arguments passed to other methods (currently not used).
#' @exportS3Method mse payoffs
mse.payoffs <- function(object, ...){
  
  merged_result <- 
    magrittr::`%>%`(magrittr::`%>%`(object,
                                    dplyr::mutate(
                                      payoffs_cds_e = cds3_diff - predictions,
                                      payoffs_vlogs_e = vlogs3_diff - predictions, 
                                      payoffs_cds_se = (payoffs_cds_e)^2,
                                      payoffs_vlogs_se = (payoffs_vlogs_e)^2
                                    )
    ),
    dplyr::summarise(
      mse_pi_cds = mean(payoffs_cds_se), 
      mse_pi_vlogs = mean(payoffs_vlogs_se), 
      n = dplyr::n(),
      .groups = 'drop'
    ))
  
  class(merged_result) <- c("mse.payoffs", "data.frame")
  
  return(merged_result)  
}

utils::globalVariables(c("cds3_diff", "payoffs_cds_e", "payoffs_cds_se", "vlogs3_diff", "payoffs_vlogs_e", "payoffs_vlogs_se", "mse_pi_cds", "mse_pi_vlogs"))

#' Create Mean Differences from Logarithmic Viral Load Data
#'
#' This function calculates the mean differences of lograithmic viral loads across time for each individual in the dataset.
#'
#' @param vl_data A data frame of longitudinal viral load values per individual, where rows represent patients and columns represent sequential measurements across time (e.g., years or visits).
#'
#' @return An object of class `"Interaction"` with the following components:
#' \describe{
#'   \item{vlog3_diff}{Mean differences of logarithmic viral load values.}
#' }
#'
#' @export
#'
#' @examples
#' data(vl_3)
#' vl_data <- vl_3[,-1]
#' result <- vlog_diff(vl_data)
vlog_diff <- function(vl_data){
  # Validación básica
  if (!is.data.frame(vl_data)# || !is.data.frame(vl_data)
  ) {
    stop("Input must be data frames.")
  }
  
  
  # #  Log transformación
  vl_log <- dplyr::transmute(vl_data, dplyr::across(dplyr::everything(), ~ ifelse(. == 0, 0, log10(.))))
  vlog3_diff <- 
  
  magrittr::`%>%`(magrittr::`%>%`(magrittr::`%>%`(dplyr::rowwise(vl_log), dplyr::mutate(vlog_diff_values = mean(diff(stats::na.omit(dplyr::c_across(dplyr::everything())))))),
                                  dplyr::ungroup()), dplyr::pull(vlog_diff_values))
  
  vlog3_diff <- as.data.frame(vlog3_diff)  
  
  class(vlog3_diff) <- c("Interaction", "data.frame")
  return(vlog3_diff)
}

utils::globalVariables(c("vlog_diff_values"))

# `nearest_payoff()` works as expected

    Code
      nearest_payoff(-0.2, pays)
    Output
      $value
      v_I_H 
       0.15 
      
      $name
      [1] "v_I_H"
      
      attr(,"class")
      [1] "NearestPayoff" "list"         

# phen_hiv() works as expected

    Code
      phen_hiv(strategy1, strategy2, alpha, beta, gamma, theta)
    Output
        prob_00 prob_01 prob_10 prob_11 pi_v pi_V
      1       1       0       0       0    1    1

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cds_diff.R
\name{cds_diff}
\alias{cds_diff}
\title{Create Mean Standardized Differences from Longitudinal CD4 Data}
\usage{
cds_diff(cd_data)
}
\arguments{
\item{cd_data}{A data frame of longitudinal CD4 count values per individual, where rows represent patients and columns represent sequential measurements across time (e.g., years or visits).}
}
\value{
An object of class \code{"Interaction"} with the following components:
\describe{
\item{cds3_diff}{Mean standardized differences of raw CD4 count values.}
}
}
\description{
This function calculates the mean standardized differences of CD4 counts across time for each individual in the dataset.
}
\examples{
data(cd_3)
cd_data <- cd_3[,-1]
result <- cds_diff(cd_data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimate_payoffs.R
\name{estimate_payoffs}
\alias{estimate_payoffs}
\title{Estimate Payoff Parameters for HIV Phenotype Interactions}
\usage{
estimate_payoffs(object, predictions)
}
\arguments{
\item{object}{An object of class \code{InteractionClassification} containing
the data on viral load differences and CD4 counts.}

\item{predictions}{A \code{data.frame} containing predictions of viral loads or CD4 values.}
}
\description{
This function estimates the payoff parameters for HIV phenotype interactions
based on the provided classification object and predictions from a viral
load model. It calculates the mean differences in viral loads and CD4 counts,
as well as the average payoffs for each classification.
}
\examples{
set.seed(42)
data(cd_3)
cd_data <- cd_3[,-1]
cd_result <- cds_diff(cd_data)
data(vl_3)
vl_data <- vl_3[,-1]
vl_result <- vlogs_diff(vl_data)
result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
data(preds)
payoffs_results <- estimate_payoffs(result, preds)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_bb84_simulation.R
\name{run_bb84_simulation}
\alias{run_bb84_simulation}
\title{BB84 Quantum Key Distribution (QKD) Simulation}
\usage{
run_bb84_simulation(
  eavesdropping_active = FALSE,
  key_length = 1000,
  test_ratio = 0.2
)
}
\arguments{
\item{eavesdropping_active}{Logical. If \code{TRUE}, the simulation includes
an eavesdropper (Eve) who performs a measure-and-resend attack, introducing
errors. If \code{FALSE}, the simulation runs under ideal, interference-free
conditions. Default is \code{FALSE}.}

\item{key_length}{Integer. Length of the quantum key sequence to be
simulated. The default is 1000, which provides good statistical stability.}

\item{test_ratio}{Numeric. Proportion (between 0 and 1) of bits that Alice
and Bob compare to detect eavesdropping during the verification phase.
Default is 0.2.}
}
\value{
A list of class \code{"BB84Simulation"} containing:
\describe{
\item{\code{QBER}}{The observed Quantum Bit Error Rate between Alice's and Bob's bits.}
\item{\code{Sifted_Length}}{Number of bits retained after basis reconciliation.}
\item{\code{Final_Key_Length}}{Number of bits remaining after error testing.}
\item{\code{Eavesdropping}}{Logical flag indicating whether eavesdropping was active.}
}
}
\description{
Simulates the BB84 protocol for quantum key distribution (QKD), illustrating
the difference in the Quantum Bit Error Rate (QBER) between an ideal channel
(no interference) and a channel under eavesdropping (Eve). The simulation
models the encoding, transmission, and measurement of quantum bits (qubits)
following the original Bennett–Brassard (1984) protocol.
}
\details{
The BB84 protocol proceeds through the following stages:
\enumerate{
\item \strong{Preparation:} Alice generates random bits and bases and encodes photons accordingly.
\item \strong{Transmission:} Photons are sent over a quantum channel.
\item \strong{Eavesdropping (optional):} Eve intercepts each photon, measures it using a random basis,
and resends a new photon, introducing potential errors.
\item \strong{Measurement:} Bob measures the incoming photons using his own random bases.
\item \strong{Sifting:} Alice and Bob retain only bits measured with matching bases.
\item \strong{Error estimation:} A random subset of the sifted bits is compared to estimate QBER.
}

If the measured QBER exceeds a security threshold (commonly 15–25\%), it indicates
that eavesdropping has occurred and the key must be discarded. In the absence of
interference, the QBER should be close to zero.
}
\examples{
# Scenario 1: Perfect Channel (No Eavesdropping)
results_no_eve <- run_bb84_simulation(eavesdropping_active = FALSE)
cat("--- Scenario 1: No Eavesdropping ---\n")
cat(paste("Sifted Key Length:", results_no_eve$Sifted_Length, "\n"))
cat(paste("Final Key Length:", results_no_eve$Final_Key_Length, "\n"))
cat(paste("Quantum Bit Error Rate (QBER):", round(results_no_eve$QBER * 100, 2), "\%\n"))
cat("Result: Secure key established successfully.\n")

# Scenario 2: Measure-and-Resend Attack
results_with_eve <- run_bb84_simulation(eavesdropping_active = TRUE)
cat("\n--- Scenario 2: With Eavesdropping ---\n")
cat(paste("Sifted Key Length:", results_with_eve$Sifted_Length, "\n"))
cat(paste("Final Key Length:", results_with_eve$Final_Key_Length, "\n"))
cat(paste("Quantum Bit Error Rate (QBER):", round(results_with_eve$QBER * 100, 2), "\%\n"))

if (results_with_eve$QBER > 0.15) {
  cat("High QBER detected. Eavesdropping likely — key discarded.\n")
} else {
  cat("No eavesdropping detected (unlikely in theory).\n")
}

}
\references{
Bennett, C. H., & Brassard, G. (1984).
\emph{Quantum cryptography: Public key distribution and coin tossing.}
Proceedings of IEEE International Conference on Computers, Systems and Signal Processing, 175–179.

Nielsen, M. A., & Chuang, I. L. (2010).
\emph{Quantum Computation and Quantum Information.} Cambridge University Press.

Rieffel, E. G., & Polak, W. H. (2011).
\emph{Quantum Computing: A Gentle Introduction.} MIT Press.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_e91_simulation.R
\name{run_e91_simulation}
\alias{run_e91_simulation}
\title{E91 Quantum Key Distribution (QKD) Simulation}
\usage{
run_e91_simulation(
  eavesdropping_active = FALSE,
  key_length = 1000,
  noise_level = 0.5
)
}
\arguments{
\item{eavesdropping_active}{Logical. If \code{TRUE}, the entanglement is partially
destroyed, simulating an attack that forces the system toward the classical limit
where \eqn{|S| \leq 2}. Default is \code{FALSE}.}

\item{key_length}{Integer. Number of simulated EPR pairs (default = 1000).}

\item{noise_level}{Numeric. Noise level (0-1) applied when
\code{eavesdropping_active = TRUE}, reducing quantum correlation.
A value of 0.5 represents partial decoherence. Default is 0.5.}
}
\value{
A list of class \code{"E91Simulation"} containing:
\describe{
\item{\code{S_Calculated}}{Observed Bell CHSH statistic.}
\item{\code{S_Theoretical}}{Quantum theoretical value (\eqn{\approx} -2.8284).}
\item{\code{Bell_Violation}}{\code{TRUE} if \eqn{|S| > 2} (secure),
\code{FALSE} if \eqn{|S| \leq 2} (insecure).}
\item{\code{Sifted_Key_Length}}{Number of bits retained for key formation.}
\item{\code{Eavesdropping}}{Indicates if an attack was simulated.}
}
}
\description{
Simulates the E91 (Ekert 1991) quantum key distribution protocol using entangled
particle pairs (EPR) and Bell’s theorem (CHSH statistic) to ensure channel security.
Security is established when the \strong{Bell Inequality is violated}, i.e.
when \eqn{|S| > 2}, indicating quantum behavior.
}
\details{
El E91 se diferencia de BB84 en que la detección del espía es \strong{simultánea} a la generación
de la clave. Los pares de bases son separados en dos conjuntos:
\enumerate{
\item \strong{Clave Secreta:} Pares con perfecta anti-correlación (ej. a2-b1, a3-b2).
\item \strong{Test de Bell:} Pares restantes usados para calcular la esperanza $E(a_i, b_j)$ y el valor $S$.
}
Si $S$ cae por debajo de 2 (límite clásico de Bell), se concluye que el entrelazamiento ha sido
roto por Eve, y la clave debe descartarse.
}
\examples{
# Escenario 1: Canal Cuántico Seguro (No Eavesdropping)
results_secure <- run_e91_simulation(eavesdropping_active = FALSE)
cat("--- Escenario 1: Sin Eavesdropping ---\n")
cat(paste("S Calculado:", round(results_secure$S_Calculated, 4), "\n"))
cat(paste("Violación de Bell:", results_secure$Bell_Violation, "\n"))

# Escenario 2: Ataque que Rompe el Entrelazamiento
results_attack <- run_e91_simulation(eavesdropping_active = TRUE)
cat("\n--- Escenario 2: Con Ataque ---\n")
cat(paste("S Calculado:", round(results_attack$S_Calculated, 4), "\n"))
cat(paste("Violación de Bell:", results_attack$Bell_Violation, "\n"))

}
\references{
Ekert, A. K. (1991). \emph{Quantum cryptography based on Bell's theorem.}
Physical Review Letters, 67(6), 661.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mse.InteractionClassification.R
\name{mse.InteractionClassification}
\alias{mse.InteractionClassification}
\title{Mean Squared Errors for Interaction Classification}
\usage{
\method{mse}{InteractionClassification}(object, ...)
}
\arguments{
\item{object}{An object of class \code{InteractionClassification} containing
the classified data and clustering results.}

\item{...}{Additional arguments passed to other methods (currently not used).}
}
\description{
Mean squared errors (MSE) for viral load differences
and CD4 count differences by comparing the actual values with the group means
from the classification.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/InteractionClassification.R
\name{InteractionClassification}
\alias{InteractionClassification}
\title{Classify HIV phenotype interactions using k-means clustering}
\usage{
InteractionClassification(cd_result, vl_result, k = 4, ns = 100, seed = 123)
}
\arguments{
\item{cd_result}{A numeric vector of differences in CD4 T-cell counts.}

\item{vl_result}{A numeric vector of differences in log viral load.}

\item{k}{Integer. The number of clusters to use in k-means. Default is 4.}

\item{ns}{Integer. Number of random initializations for the k-means algorithm. Default is 100.}

\item{seed}{Integer. Seed for random number generation to ensure reproducibility. Default is 123.}
}
\value{
A \code{data.frame} of class \code{InteractionClassification} with three columns:
\describe{
\item{cds3_result}{The CD4 count difference for each interaction.}
\item{vlogs3_result}{The viral load difference (log scale) for each interaction.}
\item{classification}{An integer label (1 to \code{k}) indicating the interaction cluster.}
}
}
\description{
This function performs k-means clustering on the differences in viral load and CD4 counts
to classify interaction types between HIV phenotypes. It returns an object of class
\code{InteractionClassification}, a \code{data.frame} with classification labels.
}
\examples{
set.seed(42)
data(cd_3)
cd_data <- cd_3[,-1]
cd_result <- cds_diff(cd_data)
data(vl_3)
vl_data <- vl_3[,-1]
vl_result <- vlogs_diff(vl_data)
result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cd_3}
\alias{cd_3}
\title{Longitudinal CD4 Lymphocyte Counts for HIV Patients (2018-2024)}
\format{
A data frame with 176 rows and 18 variables:
\describe{
\item{ID}{Unique identifier for each patient.}
\item{cd_2018_1}{CD4 count for the first measurement in 2018.}
\item{cd_2018_2}{CD4 count for the second measurement in 2018.}
\item{cd_2019_1}{CD4 count for the first measurement in 2019.}
\item{cd_2019_2}{CD4 count for the second measurement in 2019.}
\item{cd_2020_1}{CD4 count for the first measurement in 2020.}
\item{cd_2021_1}{CD4 count for the first measurement in 2021.}
\item{cd_2021_2}{CD4 count for the second measurement in 2021.}
\item{cd_2021_3}{CD4 count for the third measurement in 2021.}
\item{cd_2022_1}{CD4 count for the first measurement in 2022.}
\item{cd_2022_2}{CD4 count for the second measurement in 2022.}
\item{cd_2022_3}{CD4 count for the third measurement in 2022.}
\item{cd_2023_1}{CD4 count for the first measurement in 2023.}
\item{cd_2023_2}{CD4 count for the second measurement in 2023.}
\item{cd_2023_3}{CD4 count for the third measurement in 2023.}
\item{cd_2024_1}{CD4 count for the first measurement in 2024.}
\item{cd_2024_2}{CD4 count for the second measurement in 2024.}
\item{cd_2024_3}{CD4 count for the third measurement in 2024.}
}
}
\source{
Clinical data from Hospital Vicente Guerrero, IMSS, HIV Clinic.
}
\usage{
cd_3
}
\description{
Contains longitudinal measurements of CD4 lymphocyte counts for
176 patients living with HIV, recorded over the period from 2018 to 2024.
CD4 counts are a critical indicator of immune function, used to monitor the
progression of HIV and the effectiveness of treatments. Measurements were
taken at various points throughout the study, with some missing values due
to unavailable data for specific patients at certain times.
}
\details{
.
CD4 counts are used to monitor immune system health in individuals with
HIV. A lower CD4 count often indicates a weakened immune system, whereas
higher counts suggest a stronger immune response. Some values are missing,
indicating no measurement was taken for a particular patient at that time.
}
\examples{
# Load the dataset
data(cd_3)

# Summarize CD4 counts for the year 2021
summary(cd_3[, c("cd_2021_1", "cd_2021_2", "cd_2021_3")])

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phen_hiv.R
\name{phen_hiv}
\alias{phen_hiv}
\title{Calculate Final State and Payoffs in Quantum Game}
\usage{
phen_hiv(strategy1, strategy2, alpha, beta, gamma, theta)
}
\arguments{
\item{strategy1}{A 2x2 matrix representing the strategy of player 1.}

\item{strategy2}{A 2x2 matrix representing the strategy of player 2.}

\item{alpha}{A numeric value representing the payoff for outcome |00>.}

\item{beta}{A numeric value representing the payoff for outcome |01>.}

\item{gamma}{A numeric value representing the payoff for outcome |10>.}

\item{theta}{A numeric value representing the payoff for outcome |11>.}
}
\description{
This function calculates the final quantum state and expected payoffs for two
players in a quantum game based on their strategies. The function uses quantum
gates and unitary transformations to simulate the game dynamics.
}
\examples{
strategy1 <- diag(2) # Identity matrix for strategy 1
strategy2 <- diag(2) # Identity matrix for strategy 2
alpha <- 1
beta <- 0.5
gamma <- 2
theta <- 0.1
result <- phen_hiv(strategy1, strategy2, alpha, beta, gamma, theta)

}
\references{
Özlüer Başer, B. (2022). "Analyzing the competition of HIV-1 phenotypes with quantum game theory".
Gazi University Journal of Science, 35(3), 1190--1198. \doi{10.35378/gujs.772616}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.InteractionClassification.R,
%   R/summary.payoffs.R
\name{summary}
\alias{summary}
\title{Summarize an InteractionClassification object}
\usage{
summary(object, ...)

summary(object, ...)
}
\arguments{
\item{object}{A payoffs object.}

\item{...}{Additional arguments (not used).}
}
\value{
A \code{data.frame} with one row per interaction cluster and the following columns:
\describe{
\item{classification}{Cluster label (as factor).}
\item{cds_diff_mean}{Mean of CD4 differences in the group.}
\item{vlogs_diff_mean}{Mean of viral load differences in the group.}
\item{n}{Number of observations in the group.}
}
}
\description{
Computes summary statistics by classification group from an object of class
\code{InteractionClassification}, including mean differences in viral load and CD4 counts,
and the number of observations per cluster.

This function summarizes the payoffs object by classification.
}
\examples{
set.seed(42)
data(cd_3)
cd_data <- cd_3[,-1]
cd_result <- cds_diff(cd_data)
data(vl_3)
vl_data <- vl_3[,-1]
vl_result <- vlogs_diff(vl_data)
result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
summary(result)
set.seed(42)
data(cd_3)
cd_data <- cd_3[,-1]
cd_result <- cds_diff(cd_data)
data(vl_3)
vl_data <- vl_3[,-1]
vl_result <- vlogs_diff(vl_data)
result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
data(preds)
payoffs_results <- estimate_payoffs(result, preds)
summary(payoffs_results)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.InteractionClassification.R
\name{summary.InteractionClassification}
\alias{summary.InteractionClassification}
\title{Summarize an InteractionClassification object}
\usage{
\method{summary}{InteractionClassification}(object, ...)
}
\arguments{
\item{object}{An object of class \code{InteractionClassification} returned by the
\code{\link[=InteractionClassification]{InteractionClassification()}} function.}

\item{...}{Additional arguments passed to other methods (currently not used).}
}
\description{
Computes summary statistics by classification group from an object of class
\code{InteractionClassification}, including mean differences in viral load and CD4 counts,
and the number of observations per cluster.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nearest_payoff.R
\name{nearest_payoff}
\alias{nearest_payoff}
\title{Find Nearest Payoff}
\usage{
nearest_payoff(vl_diff, payoffs_list)
}
\arguments{
\item{vl_diff}{Numeric value representing the viral load difference for which the
nearest payoff will be found.}

\item{payoffs_list}{A named list of payoff values, where the names correspond to
specific payoffs and the values are the associated payoff values.}
}
\description{
This function computes the nearest simulated payoff from a given list of payoffs
based on a viral load difference (vl_diff). It returns both the nearest payoff value
and its corresponding payoff name.
}
\examples{
 I <- diag(2)
 H <- 1 / sqrt(2) * matrix(c(1, 1, 1, -1), 2, 2)
 Z <- diag(c(1, -1))
 gates <- list(I = I, H = H, Z = Z)
 alpha <- 0.3; beta <- 0.1; gamma <- 0.5; theta <- 0.2
 alpha2 <- 0.35; beta2 <- 0.15; gamma2 <- 0.6; theta2 <- 0.25
 pays <- payoffs_list(gates, alpha, beta, gamma, theta, alpha2, beta2, gamma2, theta2)
 nearest_payoff(-0.2, pays)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/payoffs_list.R
\name{payoffs_list}
\alias{payoffs_list}
\title{Compute Payoff Values for Quantum HIV Phenotype Interactions}
\usage{
payoffs_list(gates, alpha, beta, gamma, theta, alpha2, beta2, gamma2, theta2)
}
\arguments{
\item{gates}{A named list of 2x2 unitary matrices representing quantum strategies (e.g., I, H, Z).}

\item{alpha}{Numeric scalar, payoff coefficient for phenotype \code{v} when both play \code{0}.}

\item{beta}{Numeric scalar, payoff coefficient for phenotype \code{v} when \code{v} plays \code{0}, \code{V} plays \code{1}.}

\item{gamma}{Numeric scalar, payoff coefficient for phenotype \code{v} when \code{v} plays \code{1}, \code{V} plays \code{0}.}

\item{theta}{Numeric scalar, payoff coefficient for phenotype \code{v} and \code{V} when both play \code{1}.}

\item{alpha2}{Numeric scalar, alternate value of \code{alpha} for phenotype \code{v} in a second scenario.}

\item{beta2}{Numeric scalar, alternate value of \code{beta} for phenotype \code{v} in a second scenario.}

\item{gamma2}{Numeric scalar, alternate value of \code{gamma} for phenotype \code{v} in a second scenario.}

\item{theta2}{Numeric scalar, alternate value of \code{theta} for phenotype \code{v} in a second scenario.}
}
\description{
Computes payoff values for all pairwise combinations of quantum gate strategies provided in
a named list. For each pair, the function calculates the payoffs for both phenotypes \code{v} and \code{V}
using two different sets of payoff parameters.
}
\examples{
I <- diag(2)
H <- 1 / sqrt(2) * matrix(c(1, 1, 1, -1), 2, 2)
Z <- diag(c(1, -1))
gates <- list(I = I, H = H, Z = Z)
payoffs <- payoffs_list(gates, 1, 0.5, 0.3,0.2, 1.5, 0.6, 0.7, 0.8)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mse.payoffs.R
\name{mse.payoffs}
\alias{mse.payoffs}
\title{Mean Squared Errors for Payoff Predictions}
\usage{
\method{mse}{payoffs}(object, ...)
}
\arguments{
\item{object}{An object of class \code{payoffs}.}

\item{...}{Additional arguments passed to other methods (currently not used).}
}
\description{
Computes the mean squared error (MSE) between observed CD4 and viral load differences
and their corresponding predicted payoff values within each interaction classification.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vlog_diff.R
\name{vlog_diff}
\alias{vlog_diff}
\title{Create Mean Differences from Logarithmic Viral Load Data}
\usage{
vlog_diff(vl_data)
}
\arguments{
\item{vl_data}{A data frame of longitudinal viral load values per individual, where rows represent patients and columns represent sequential measurements across time (e.g., years or visits).}
}
\value{
An object of class \code{"Interaction"} with the following components:
\describe{
\item{vlog3_diff}{Mean differences of logarithmic viral load values.}
}
}
\description{
This function calculates the mean differences of lograithmic viral loads across time for each individual in the dataset.
}
\examples{
data(vl_3)
vl_data <- vl_3[,-1]
result <- vlog_diff(vl_data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.payoffs.R
\name{summary.payoffs}
\alias{summary.payoffs}
\title{Summarize Payoffs}
\usage{
\method{summary}{payoffs}(object, ...)
}
\arguments{
\item{object}{A payoffs object.}

\item{...}{Additional arguments (not used).}
}
\description{
This function summarizes the payoffs object by classification.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_corr}
\alias{data_corr}
\title{Biomarker Correlation Dataset for Hamiltonian Construction}
\format{
A data frame with 12 rows and 30 variables:
\describe{
\item{ACMSD}{Amino acid metabolism biomarker.}
\item{ADA}{Nucleotide metabolism enzyme.}
\item{ADK}{Adenosine kinase (nucleotide metabolism).}
\item{CCL13}{Chemokine involved in immune signaling.}
\item{CCL4}{Inflammatory chemokine.}
\item{CCL5}{Immune signaling chemokine.}
\item{CD14}{Innate immune receptor.}
\item{CD163}{Macrophage activation marker.}
\item{CPS1}{Urea cycle enzyme.}
\item{CTSD}{Lysosomal protease.}
\item{CTSL}{Cathepsin L (antigen processing).}
\item{FAH}{Fumarylacetoacetate hydrolase.}
\item{FASN}{Fatty acid synthase.}
\item{HADH}{Beta-oxidation enzyme.}
\item{HK2}{Hexokinase 2 (glycolysis).}
\item{IDO1}{Tryptophan metabolism enzyme.}
\item{IL6}{Inflammatory cytokine.}
\item{IMPDH2}{Purine biosynthesis enzyme.}
\item{LAT}{T-cell activation linker protein.}
\item{LDHB}{Lactate dehydrogenase B.}
\item{LTC4S}{Leukotriene synthesis enzyme.}
\item{MTF1}{Metal regulatory transcription factor.}
\item{NT5E}{Ecto-5'-nucleotidase.}
\item{PKM}{Pyruvate kinase (glycolysis).}
\item{PTGES}{Prostaglandin synthesis enzyme.}
\item{PTGS2}{Cyclooxygenase-2 (inflammation).}
\item{RPTOR}{mTOR pathway regulator.}
\item{TDO2}{Tryptophan metabolism enzyme.}
\item{ZNF708}{Zinc finger transcription factor.}
\item{ZNF93}{Zinc finger regulatory protein.}
}
}
\usage{
data(data_corr)
}
\description{
A numeric data frame containing a curated panel of biomarkers used to
estimate correlation-driven coefficients for a variational Hamiltonian
in a 3-qubit quantum model.
}
\details{
The Hamiltonian is defined as:

\deqn{
\hat{H} = \sum_{k=1}^K c_k P_k
}

where:
\itemize{
\item \eqn{P_k} are Pauli strings from the 3-qubit Pauli group
\item \eqn{c_k} are coefficients estimated via Pearson correlation
between selected biomarkers
}

The biomarkers in \code{data_corr} are grouped into biological axes:

\itemize{
\item \strong{Amino Acid Metabolism:} IDO1, TDO2, ACMSD, CPS1, FAH
\item \strong{Immune Signaling:} IL6, CCL4, CCL5, CCL13, NT5E
\item \strong{Antigen Processing:} CD14, CD163, LAT, CTSL, CTSD
\item \strong{Fatty Acid / Lipid:} HADH, FASN, LTC4S, PTGS2, PTGES
\item \strong{Aerobic / Nucleotide:} LDHB, PKM, IMPDH2, ADA, ADK, HK2
\item \strong{Regulatory / Epigenetic:} ZNF93, MTF1, RPTOR, ZNF708, CAD
}

These features define a high-dimensional correlation structure used to
construct patient-specific or cohort-level Hamiltonians.
}
\examples{
data(data_corr)
dim(data_corr)

}
\references{
Brown, A. J., et al. (2024). Metabolic reprogramming of human macrophages during Mycobacterium tuberculosis infection under HIV exposure. Gene Expression Omnibus (GEO), accession GSE314344.
Baluku, J. B., et al. (2023). DNA methylation profiling of people living with HIV stratified by tuberculosis status. Gene Expression Omnibus (GEO), accession GSE304107.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{vl_3}
\alias{vl_3}
\title{Longitudinal Viral Load Values for HIV Patients (2018-2024)}
\format{
A data frame with 176 rows and 18 variables:
\describe{
\item{ID}{Unique identifier for each patient.}
\item{vl_2018_1}{Viral load for the first measurement in 2018.}
\item{vl_2018_2}{Viral load for the second measurement in 2018.}
\item{vl_2019_1}{Viral load for the first measurement in 2019.}
\item{vl_2019_2}{Viral load for the second measurement in 2019.}
\item{vl_2020_1}{Viral load for the first measurement in 2020.}
\item{vl_2021_1}{Viral load for the first measurement in 2021.}
\item{vl_2021_2}{Viral load for the second measurement in 2021.}
\item{vl_2021_3}{Viral load for the third measurement in 2021.}
\item{vl_2022_1}{Viral load for the first measurement in 2022.}
\item{vl_2022_2}{Viral load for the second measurement in 2022.}
\item{vl_2022_3}{Viral load for the third measurement in 2022.}
\item{vl_2023_1}{Viral load for the first measurement in 2023.}
\item{vl_2023_2}{Viral load for the second measurement in 2023.}
\item{vl_2023_3}{Viral load for the third measurement in 2023.}
\item{vl_2024_1}{Viral load for the first measurement in 2024.}
\item{vl_2024_2}{Viral load for the second measurement in 2024.}
\item{vl_2024_3}{Viral load for the third measurement in 2024.}
}
}
\source{
Clinical data from Hospital Vicente Guerrero, IMSS, HIV Clinic.
}
\usage{
vl_3
}
\description{
Contains longitudinal measurements of viral load for 176
patients from 2018 to 2024. Viral load is a critical marker used to monitor
the effectiveness of HIV treatment by measuring the amount of HIV RNA in the
blood.
}
\details{
The viral load measurements provide insight into the patient's response to
antiretroviral therapy (ART). Lower viral load values, especially
undetectable levels, indicate better control of the infection. Missing
values indicate that no viral load measurement was available for that patient
at that specific time.
}
\examples{
\dontrun{
# Load the dataset
data(vl_3)

# Summarize viral loads for the year 2021
summary(vl_3[, c("cd_2021_1", "cd_2021_2", "cd_2021_3")])
}

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_enc}
\alias{data_enc}
\title{Angle-Encoding Dataset for 3-Qubit Quantum State Preparation}
\format{
A data frame with 12 rows and 6 variables:
\describe{
\item{CD14}{Expression level of CD14 (innate immune receptor; phi2 angle).}
\item{HADH}{Expression level of HADH (beta-oxidation marker; theta3 angle).}
\item{IDO1}{Expression level of IDO1 (tryptophan metabolism; phi1 angle).}
\item{IL6}{Expression level of IL6 (inflammatory signaling; theta1 angle).}
\item{LDHB}{Expression level of LDHB (glycolytic flux; phi3 angle).}
\item{TDO2}{Expression level of TDO2 (systemic metabolic repression; theta2 angle).}
}
}
\usage{
data(data_enc)
}
\description{
A numeric data frame containing patient-level biomarker expression values
used for angle encoding into a 3-qubit quantum state. Each row corresponds
to a sample (patient), and each column corresponds to a biomarker mapped
to a rotation angle in the quantum circuit.
}
\details{
The encoding follows a 3-qubit angle encoding scheme:

\deqn{
|\psi_j\rangle = \bigotimes_{i=1}^3 R_y(\theta_{j,i}) R_z(\phi_{j,i}) |0\rangle
}

where each qubit is parameterized by two biomarkers:

\itemize{
\item \strong{Qubit 1:} \code{IL6} (\eqn{\theta_1}), \code{IDO1} (\eqn{\phi_1})
\item \strong{Qubit 2:} \code{TDO2} (\eqn{\theta_2}), \code{CD14} (\eqn{\phi_2})
\item \strong{Qubit 3:} \code{HADH} (\eqn{\theta_3}), \code{LDHB} (\eqn{\phi_3})
}

These biomarkers capture key immunometabolic axes, including inflammation,
tryptophan metabolism, innate immune sensing, and metabolic reprogramming.

Prior to encoding, values are typically normalized to angular ranges
(e.g., \eqn{[0, 2\pi]}) using dataset-specific min–max scaling.
}
\examples{
data(data_enc)
str(data_enc)

}
\references{
Brown, A. J., et al. (2024). Metabolic reprogramming of human macrophages during Mycobacterium tuberculosis infection under HIV exposure. Gene Expression Omnibus (GEO), accession GSE314344.
Baluku, J. B., et al. (2023). DNA methylation profiling of people living with HIV stratified by tuberculosis status. Gene Expression Omnibus (GEO), accession GSE304107.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mse.InteractionClassification.R,
%   R/mse.payoffs.R
\name{mse}
\alias{mse}
\title{Mean Squared Errors for Interaction Classification}
\usage{
mse(object, ...)

mse(object, ...)
}
\arguments{
\item{object}{An object of class \code{payoffs}.}

\item{...}{Additional arguments passed to other methods (currently not used).}
}
\value{
A \code{data.frame} containing the MSE for CD4 count differences (\code{mse_cds_diff})
and (\code{mse_vlogs_diff}) for viral load differences.
}
\description{
Mean squared errors (MSE) for viral load differences
and CD4 count differences by comparing the actual values with the group means
from the classification.

Computes the mean squared error (MSE) between observed CD4 and viral load differences
and their corresponding predicted payoff values within each interaction classification.
}
\examples{
set.seed(42)
data(cd_3)
cd_data <- cd_3[,-1]
cd_result <- cds_diff(cd_data)
data(vl_3)
vl_data <- vl_3[,-1]
vl_result <- vlogs_diff(vl_data)
result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
mse(result)

set.seed(42)
data(cd_3)
cd_data <- cd_3[,-1]
cd_result <- cds_diff(cd_data)
data(vl_3)
vl_data <- vl_3[,-1]
vl_result <- vlogs_diff(vl_data)
result <- InteractionClassification(cd_result = cd_result, vl_result = vl_result)
data(preds)
payoffs_results <- estimate_payoffs(result, preds)
mse(payoffs_results)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{qphen}
\alias{qphen}
\title{Quantum Phenotype Interactions in HIV Model}
\format{
A data frame with 176 rows and 24 variables:
\describe{
\item{id}{(double) Unique identifier for each observation.}
\item{vl_diff}{(double) Difference in viral load (log scale) between time points.}
\item{cd_diff}{(double) Difference in CD4 count between time points.}
\item{vlogs_diff_mean}{(double) Mean difference of viral loads across the dataset.}
\item{cds_diff_mean}{(double) Mean difference of CD4 counts across the dataset.}
\item{n}{(double) Number of cases in each interaction cluster.}
\item{payoffs}{(double) Computed payoff for the phenotype interaction.}
#'   \item{payoffs_b}{(double) Alternative computed payoff.}
\item{nearest_payoff}{(double) Closest estimated payoff value.}
\item{classification_2}{(double) Cluster assignment for phenotype interactions (second clustering method).}
\item{classification_3}{(double) Cluster assignment for phenotype interactions (third clustering method).}
\item{classification_4}{(double) Cluster assignment for phenotype interactions (fourth clustering method).}
\item{phen_1}{(double) Phenotype type (\code{v} or \code{V}).}
\item{str1_2}{(double) Strategy of the first phenotype using \code{X}, \code{T}, or \code{H} gate (binary encoding).}
\item{str1_3}{(double) Alternative strategy of the first phenotype.}
\item{str2_2}{(double) Strategy of the second phenotype using \code{H}, \code{Id}, \code{S}, \code{T}, \code{X}, \code{Y}, or \code{Z} gate (binary encoding).}
\item{str2_3}{(double) Alternative strategy of the second phenotype.}
\item{str2_4}{(double) Alternative strategy of the second phenotype.}
\item{str2_5}{(double) Alternative strategy of the second phenotype.}
\item{str2_6}{(double) Alternative strategy of the second phenotype.}
\item{str2_7}{(double) Alternative strategy of the second phenotype.}
\item{batch_1}{(double) Indicates whether predictions were made on full data or batch data.}
\item{TB_1}{(double) Indicator for tuberculosis presence (1 = TB, 0 = no TB).}
\item{GR_1}{(double) Indicator for genoresistance presence (1 = resistant, 0 = non-resistant).}
}
}
\usage{
data(qphen)
}
\description{
The \code{qphen} dataset contains 176 observations and 24 variables, representing classified phenotype interactions in a quantum game-theoretic model of HIV phenotypes.
The data includes CD4 and viral load differences, quantum game strategies, classification clusters, and tuberculosis/genoresistance indicators.
}
\examples{
data(qphen)
head(qphen)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{preds2}
\alias{preds2}
\title{Batched Predictions for Longitudinal Viral Load Values for HIV Patients (2018-2024)}
\format{
An object of class \code{spec_tbl_df} (inherits from \code{tbl_df}, \code{tbl}, \code{data.frame}) with 176 rows and 1 columns.
}
\source{
Clinical data from Hospital Vicente Guerrero, IMSS, HIV Clinic.
}
\usage{
preds2
}
\description{
Contains batched predictions of longitudinal viral load values for 176
patients from 2018 to 2024.
}
\examples{
data(preds2)
head(preds2)

}
\keyword{datasets}
